from django_serializer.exceptions import SerializerFieldException


class Field:
    def __init__(self, required=True, default=None, name=None):
        self.default = default
        self.required = required
        self.name = name

    def serialize(self, **kwargs):
        if 'field_value' not in kwargs:
            if not self.required:
                serialized = self.default
            else:
                raise SerializerFieldException('Field missed or has no default value')
        else:
            serialized = kwargs['field_value']
            serialized = self.serialization_handler(serialized)

        return serialized

    def serialization_handler(self, value):
        pass


class IntegerField(Field):
    def serialization_handler(self, value):
        return int(value) if value is not None else None


class BooleanField(Field):
    def serialization_handler(self, value):
        return int(value) if value is not None else None


class CharField(Field):
    def serialization_handler(self, value):
        return str(value) if value is not None else None


class DateField(Field):
    def serialization_handler(self, value):
        from django.utils.dateformat import format
        return format(value, 'U') if value is not None else None


class DateTimeField(Field):
    def serialization_handler(self, value):
        from django.utils.dateformat import format
        return format(value, 'U') if value is not None else None


class TimeField(Field):
    def serialization_handler(self, value):
        return str(value) if value is not None else None


class FloatField(Field):
    def serialization_handler(self, value):
        return float(value) if value is not None else None


class ModelField(Field):
    def serialization_handler(self, value):
        return value if value is not None else None


# class PointField(Field):
#     def serialization_handler(self, value):
#         if value is not None:
#             return {
#                 'lng': value.x,
#                 'lat': value.y
#             }


# class JSONField(Field):
#     def serialization_handler(self, value):
#         if value is not None:
#             return dict(value)


# class ArrayField(Field):
#     def serialization_handler(self, value):
#         if value is not None:
#             return list(value)


class ImageField(Field):
    def serialization_handler(self, value):
        if value:
            try:
                return value.url
            except ValueError:
                return None


class SerializerField(Field):
    def __init__(self, **kwargs):
        source = kwargs.pop('source')
        self.source = source
        self.serializer = None
        super().__init__(**kwargs)

    def serialization_handler(self, obj):
        if callable(self.source):
            return self.source(obj)
        return getattr(self.serializer, self.source)(obj)



from django import forms
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt

from django_serializer.exceptions import ServerError


class CsrfExemptMixin:
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)


class ObjectMixin:
    model = None

    class ObjectForm(forms.Form):
        id = forms.IntegerField()

    def get_args_form(self):
        return self.ObjectForm

    def get_object(self):
        try:
            return self.model.objects.get(pk=self.request_args['id'])
        except self.model.DoesNotExist:
            raise ServerError(ServerError.NOT_FOUND)
        except (ValueError, KeyError):
            raise ServerError(ServerError.BAD_REQUEST)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['instance'] = self.get_object()
        return kwargs


class SerializerMixin:
    serializer = None

    def get_serializer_class(self, obj=None):
        return self.serializer

    def get_serializer_kwargs(self, obj, **kwargs):
        serializer_kwargs = {
            'obj': obj
        }
        serializer_kwargs.update(kwargs)
        return serializer_kwargs

    def get_serializer(self, obj, **kwargs):
        serializer_class = self.get_serializer_class(obj)
        return serializer_class(**self.get_serializer_kwargs(obj, **kwargs))

    def response_middleware(self, response):
        if self.get_serializer_class(response):
            serializer = self.get_serializer(response)
            response = serializer.serialize()
        return response


class FormMixin:
    form_class = None

    def get_form_kwargs(self):
        kwargs = {
            'data': self.request.GET,
        }

        if self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data': self.request_body,
                'files': self.request.FILES,
            })
        return kwargs

    def get_form_class(self):
        return self.form_class

    def get_form(self):
        """
        Returns an instance of the form to be used in this view.
        """
        form_class = self.get_form_class()
        return form_class(**self.get_form_kwargs())


class ListMixin:
    model = None
    paginator = None

    def get_paginator_class(self):
        return self.paginator

    def get_paginator_kwargs(self):
        return {
            'object_list': self.get_queryset(),
            'arguments': self.request.GET
        }

    def get_paginator(self):
        if hasattr(self, '_paginator'):
            return getattr(self, '_paginator')
        if self.paginator:
            setattr(self, '_paginator', self.get_paginator_class()(**self.get_paginator_kwargs()))
            return getattr(self, '_paginator')

    def get_queryset(self):
        return self.model.objects.all().order_by('id')

    def get_serializer_kwargs(self, obj, **kwargs):
        return super().get_serializer_kwargs(obj, **{'multiple': True})

    def response_middleware(self, response):
        response = super().response_middleware(response)
        paginator = self.get_paginator()
        if paginator:
            response = {
                'count': self.get_paginator().count,
                'list': response
            }
        return response



from django import forms
from django.core.exceptions import ValidationError
import json


class ObjectField(forms.Field):
    default_error_messages = {
        'invalid_type': 'Object is incorrect'
    }

    @staticmethod
    def get_field_problems(form):
        field_problems = []
        for field, error in form.errors.items():
            for data_item in error.data:
                for message in data_item.messages:
                    field_problems.append('{}: {}'.format(field, message))
        return field_problems

    def form_validator(self, value):
        if self.multiple:
            for item in value:
                form = self.form_class(data=item)
                if not form.is_valid():
                    raise ValidationError(self.get_field_problems(form), code='invalid_form')
                else:
                    self._validate_forms.append(form)
        else:
            form = self.form_class(data=value)
            if not form.is_valid():
                raise ValidationError(self.get_field_problems(form), code='invalid_form')
            self._validate_forms.append(form)

    def __init__(self, form_class, multiple=False, *args, **kwargs):
        validators = kwargs.pop('validators', [])
        if isinstance(validators, tuple):
            validators += (self.form_validator,)
        else:
            validators.append(self.form_validator)
        self.form_class = form_class
        self.multiple = multiple
        self._validate_forms = []
        super().__init__(*args, validators=validators, **kwargs)

    def to_python(self, value):
        value = super().to_python(value)
        self._validate_forms.clear()
        if isinstance(value, dict):
            return value
        elif isinstance(value, str):
            return json.loads(value)
        else:
            return value

    def validate(self, value):
        super().validate(value)

        if value is None:
            return

        if self.multiple:
            if not isinstance(value, list):
                raise ValidationError(self.error_messages['invalid_type'], code='invalid_type')
            if any([not isinstance(item, dict) for item in value]):
                raise ValidationError(self.error_messages['invalid_type'], code='invalid_type')
        else:
            if not isinstance(value, dict):
                raise ValidationError(self.error_messages['invalid_type'], code='invalid_type')

    def clean(self, value):
        super().clean(value)
        if self.multiple:
            return [form.cleaned_data for form in self._validate_forms]
        if len(self._validate_forms) > 0:
            return self._validate_forms[0].cleaned_data



class ModelException(Exception):
    pass


class MissEntityTypeException(ModelException):
    pass


class DuplicateEntityTypeException(ModelException):
    pass


class SerializerException(Exception):
    pass


class SerializerFieldException(SerializerException):
    pass


class MetaSerializerException(SerializerException):
    pass


class MappingSerializerException(SerializerException):
    pass


class ServerResponseStatus(object):
    def __init__(self, status_alias, status_description, http_code=200):
        self.alias = status_alias
        self.description = status_description
        self.http_code = http_code


class BaseViewException(Exception):
    BAD_REQUEST = ServerResponseStatus('bad_request', 'Bad request', 400)
    INVALID_PARAMETER = ServerResponseStatus('invalid_param', 'Invalid parameter', 400)
    AUTH_NOT_REQUIRED = ServerResponseStatus('auth_not_required', 'Authentication not required', 400)

    AUTH_REQUIRED = ServerResponseStatus('auth_required', 'Authentication required', 401)
    ACCESS_DENIED = ServerResponseStatus('access_denied', 'Access denied', 401)

    FORBIDDEN = ServerResponseStatus('forbidden', 'Forbidden', 403)
    INVALID_CREDENTIALS = ServerResponseStatus('invalid_credentials', 'Invalid credentials', 403)
    ROLE_FORBIDDEN = ServerResponseStatus('role_forbidden', 'For your role access denied', 403)
    ACCOUNT_INACTIVE = ServerResponseStatus('account_not_active', 'Account is not active', 403)

    NOT_FOUND = ServerResponseStatus('not_found', 'Not found', 404)

    NOT_IMPLEMENTED = ServerResponseStatus('not_implemented', 'Not implemented', 405)

    TOO_MANY_REQUESTS = ServerResponseStatus('too_many_requests', 'Too many requests', 429)

    MIX_FIELDS_FILTER = ServerResponseStatus('mix_fields_filter', 'Cannot have a mix of inclusion and exclusion', 500)
    STORAGE_ERROR = ServerResponseStatus('storage_error', 'Storage error', 500)
    INTERNAL_SERVER_ERROR = ServerResponseStatus('internal_error', 'Internal server error', 500)
    UNKNOWN = ServerResponseStatus('unknown_error', 'Unknown error', 500)
    FILE_SAVE_ERROR = ServerResponseStatus('file_save_error', "Can't save data", 500)

    def __init__(self, status, description=None, field_problems=None):
        if not isinstance(status, ServerResponseStatus):
            status = self.UNKNOWN

        self.status = status
        self.description = self.status.description
        if isinstance(description, str):
            self.description = description

        self.field_problems = field_problems

    def get_http_code(self):
        return self.status.http_code

    def get_description(self):
        return self.description

    def get_alias(self):
        return self.status.alias

    def get_field_problems(self):
        return self.field_problems


class FormException(BaseViewException):
    def __init__(self, form):
        self.form = form
        status = self.BAD_REQUEST
        field_problems = {}
        for field, error in form.errors.items():
            problems = []
            for data_item in error.data:
                for message in data_item.messages:
                    problems.append(message)
            field_problems[field] = problems
        super().__init__(status, field_problems=field_problems)


class ServerError(BaseViewException):
    pass



from django import forms

from django_serializer.exceptions import FormException


class Paginator(object):
    form = None

    def __init__(self, object_list, arguments):
        self.object_list = object_list
        self._arguments = arguments
        self._count = None
        self.validated_arguments = None

    def get_form(self):
        return self.get_form_class()(**self.get_form_kwargs())

    def get_form_class(self):
        return self.form

    def get_form_kwargs(self):
        return {'data': self._arguments}

    def validate_arguments(self):
        form = self.get_form()
        if form.is_valid():
            self.validated_arguments = form.cleaned_data
        else:
            raise FormException(form)

    def _get_count(self):
        if self._count is None:
            try:
                self._count = self.object_list.count()
            except (AttributeError, TypeError):
                self._count = len(self.object_list)
        return self._count
    count = property(_get_count)

    def page(self):
        if self.validated_arguments is None:
            self.validate_arguments()


class LimitOffsetPaginator(Paginator):
    LIMIT = 20
    OFFSET = 0

    class LimitOffsetForm(forms.Form):
        limit = forms.IntegerField(min_value=1, max_value=100, required=False)
        offset = forms.IntegerField(min_value=0, required=False)

    form = LimitOffsetForm

    def page(self):
        super().page()
        bottom = self.validated_arguments['offset'] or self.OFFSET
        top = bottom + (self.validated_arguments['limit'] or self.LIMIT)
        return self.object_list[bottom:top]



from django_serializer.exceptions import ServerError


class PermissionsModelMixin:
    class Permission:
        R = 1       # read permission
        W = 1 << 1  # write permission
        D = 1 << 2  # delete permission

    owner_permission = (Permission.R, Permission.W, Permission.D)
    group_permission = (Permission.R, Permission.W)
    authorized_permission = (Permission.R, )
    unauthorized_permission = ()

    def is_owner(self, user):
        pass

    def in_group(self, user):
        pass


class PermissionsMixin(PermissionsModelMixin):
    class DefaultPermissions(PermissionsModelMixin):
        authorized_permission = (PermissionsModelMixin.Permission.R, PermissionsModelMixin.Permission.W)

    owner_permission = ()
    group_permission = ()
    authorized_permission = ()
    unauthorized_permission = ()

    def get_permissions_object(self):
        conditions = (
            bool(self.owner_permission),
            bool(self.group_permission),
            bool(self.authorized_permission),
            bool(self.unauthorized_permission),
        )

        if any(conditions):
            obj = self
        elif hasattr(self, 'get_object'):
            obj = self.get_object()
        else:
            obj = self.DefaultPermissions()

        return obj

    def get_permissions(self, user):
        if hasattr(self, '_permissions'):
            return getattr(self, '_permissions')

        obj = self.get_permissions_object()

        if user.is_authenticated and obj.is_owner(user):
            permissions = obj.owner_permission
        elif user.is_authenticated and obj.in_group(user):
            permissions = obj.group_permission
        elif user.is_authenticated:
            permissions = obj.authorized_permission
        else:
            permissions = obj.unauthorized_permission

        setattr(self, '_permissions', permissions)

        return permissions

    def check_permission(self, user, permission):
        if user.is_superuser:
            return
        if permission not in self.get_permissions(user):
            raise ServerError(ServerError.ACCESS_DENIED)

    def check_r_permission(self, user):
        self.check_permission(user, PermissionsModelMixin.Permission.R)

    def check_w_permission(self, user):
        self.check_permission(user, PermissionsModelMixin.Permission.W)

    def check_d_permission(self, user):
        self.check_permission(user, PermissionsModelMixin.Permission.D)



from django.conf import settings
from django.db import models

from django_serializer.model.base import EntityField
from django_serializer.exceptions import MetaSerializerException, MappingSerializerException
from django_serializer.serializer.fields import (
    Field,
    IntegerField,
    BooleanField,
    CharField,
    DateField,
    DateTimeField,
    FloatField,
    ModelField,
    SerializerField,
    TimeField,
    ImageField,
)

SERIALIZER_FIELD_MAPPING = {
    models.AutoField: IntegerField,
    models.BigIntegerField: IntegerField,
    models.BooleanField: BooleanField,
    models.CharField: CharField,
    models.CommaSeparatedIntegerField: CharField,
    models.DateField: DateField,
    models.DateTimeField: DateTimeField,
    models.TimeField: TimeField,
    models.DecimalField: IntegerField,
    models.EmailField: CharField,
    models.ForeignKey: ModelField,
    models.OneToOneField: ModelField,
    models.FloatField: FloatField,
    models.IntegerField: IntegerField,
    models.PositiveIntegerField: IntegerField,
    models.PositiveSmallIntegerField: IntegerField,
    models.SmallIntegerField: IntegerField,
    models.TextField: CharField,
    EntityField: IntegerField,
    models.ImageField: ImageField,
    models.URLField: CharField
}
EXTRA_SERIALIZER_FIELD_MAPPING = getattr(
    settings, 'SERIALIZER_FIELD_MAPPING', None
)
if EXTRA_SERIALIZER_FIELD_MAPPING:
    assert isinstance(EXTRA_SERIALIZER_FIELD_MAPPING, dict), \
        'SERIALIZER_FIELD_MAPPING should be dict'
    SERIALIZER_FIELD_MAPPING.update(EXTRA_SERIALIZER_FIELD_MAPPING)


class SerializerMeta(type):
    def __new__(mcs, name, bases, attrs):
        serializer_attrs = attrs.copy()

        for base_class in bases:
            serializer_attrs.update(base_class.__dict__)
            for c in reversed(base_class.__mro__):
                serializer_attrs.update(c.__dict__)

        attrs['serializer_attrs'] = []

        for field_name, field in serializer_attrs.items():
            if isinstance(field, Field):
                attrs['serializer_attrs'].append((field_name, field))

        new = super(SerializerMeta, mcs).__new__(mcs, name, bases, attrs)
        return new


class Serializer(metaclass=SerializerMeta):
    def __init__(self, obj, multiple=False, dict_format=False, dict_key='id'):
        self.obj = obj
        self.multiple = multiple
        self.dict_format = dict_format
        self.dict_key = dict_key

    def _get_fields(self):
        return getattr(self, 'serializer_attrs')

    def _get_extractor(self, obj):
        def obj_extractor(inner_obj, field):
            try:
                return getattr(inner_obj, field)
            except AttributeError:
                return getattr(self, field)(inner_obj)

        def dict_extractor(inner_obj, field):
            try:
                return inner_obj[field]
            except KeyError:
                return getattr(self, field)(inner_obj)

        if isinstance(obj, dict):
            return dict_extractor
        else:
            return obj_extractor

    def _serialize_obj(self, obj):
        serialized = {}

        fields = self._get_fields()
        extractor = self._get_extractor(obj)

        for field_name, field in fields:
            try:
                if isinstance(field, SerializerField):
                    field.serializer = self
                    serialized_field_value = field.serialize(field_value=obj)
                else:
                    obj_field_value = extractor(obj, field_name)
                    serialized_field_value = field.serialize(field_value=obj_field_value)
            except AttributeError:
                serialized_field_value = field.serialize()
            serialized[field_name] = serialized_field_value

        return serialized

    def serialize(self):
        if self.obj is None:
            return None

        if self.multiple:
            if self.dict_format:
                result = {}
                for obj_item in self.obj:
                    result[getattr(obj_item, self.dict_key)] = self._serialize_obj(obj_item)
            else:
                result = []
                for obj_item in self.obj:
                    result.append(self._serialize_obj(obj_item))
        else:
            if self.dict_format:
                result = {getattr(self.obj, self.dict_key): self._serialize_obj(self.obj)}
            else:
                result = self._serialize_obj(self.obj)

        return result


class ModelSerializerMeta(SerializerMeta):
    def __new__(mcs, name, bases, attrs):
        if name == 'ModelSerializer':
            new = super(ModelSerializerMeta, mcs).__new__(mcs, name, bases, attrs)
            return new

        Meta = None
        try:
            Meta = attrs['Meta']
        except KeyError:
            for base in bases:
                if 'Meta' in base.__dict__:
                    Meta = base.__dict__['Meta']
                    break
                for c in reversed(base.__mro__):
                    if 'Meta' in c.__dict__:
                        Meta = c.__dict__['Meta']
                        break
        if not Meta:
            raise MetaSerializerException('{} serializer have not Meta'.format(name))

        try:
            model = Meta.model
        except AttributeError:
            raise MetaSerializerException('{}.Meta have not model'.format(name))

        meta_fields = getattr(Meta, 'fields', None)
        if meta_fields and not isinstance(meta_fields, (tuple, list)):
            raise MetaSerializerException('{}.Meta.fields have incorrect format'.format(name))

        meta_fields = set(meta_fields) if meta_fields else None

        meta_exclude = getattr(Meta, 'exclude', None)
        if meta_exclude and not isinstance(meta_exclude, (tuple, list)):
            raise MetaSerializerException('{}.Meta.meta_exclude have incorrect format'.format(name))

        meta_exclude = set(meta_exclude) if meta_exclude else None

        model_fields = model._meta.fields
        for model_field in model_fields:
            model_field_class = model_field.__class__

            if meta_fields is not None and model_field.attname not in meta_fields:
                continue

            if meta_exclude is not None and model_field.attname in meta_exclude:
                continue

            if hasattr(model_field_class, 'serializer'):
                serializer_field_class = model_field_class.serializer
            else:
                try:
                    serializer_field_class = SERIALIZER_FIELD_MAPPING[model_field_class]
                except KeyError:
                    raise MappingSerializerException('{} is not supported'.format(model_field_class))

            attrs[model_field.attname] = serializer_field_class()

        new = super(ModelSerializerMeta, mcs).__new__(mcs, name, bases, attrs)
        return new


class ModelSerializer(Serializer, metaclass=ModelSerializerMeta):
    pass


class MultiSerializer(Serializer):
    serializers = {}

    def get_serializer_class(self, obj, **kwargs):
        raise NotImplementedError

    def get_serializer_kwargs(self, obj, **kwargs):
        return {
            'obj': obj
        }

    def _serialize_obj(self, obj):
        serializer_class = self.get_serializer_class(obj)
        return serializer_class(**self.get_serializer_kwargs(obj)).serialize()



import json

from django.http import HttpResponse
from django.views.generic import View

from django_serializer.permissions import PermissionsMixin
from django_serializer.exceptions import (FormException, ServerError, )
from django_serializer.exceptions import BaseViewException as BVE_1
from django_serializer.exceptions import BaseViewException as BVE_2
from django_serializer.mixins import (
    CsrfExemptMixin,
    SerializerMixin,
    ObjectMixin,
    FormMixin,
    ListMixin,
)


class BaseView(View):
    args_form = None

    @property
    def request_args(self):
        return getattr(self, '_request_args', {})

    def get_args_form(self):
        return self.args_form

    @property
    def request_body(self):
        if hasattr(self, '_request_body'):
            return getattr(self, '_request_body')
        try:
            data = json.loads(self.request.body.decode('utf-8'))
        except Exception:
            data = {}
        setattr(self, '_request_body', data)
        return data

    def clean_args(self):
        if not self.get_args_form():
            return
        if hasattr(self, '_request_args'):
            return

        kwargs = {'data': self.request.GET}
        if self.request.method in ('POST', 'PUT'):
            kwargs.update({'data': self.request_body})
        form = self.get_args_form()(**kwargs)

        if form.is_valid():
            setattr(self, '_request_args', form.cleaned_data)
        else:
            raise FormException(form)

    def dispatch(self, request, *args, **kwargs):
        try:
            self.clean_args()
            response = super().dispatch(request, *args, **kwargs)
            response = self.response_middleware(response)
            return self.response_wrapper(response)
        except (BVE_1, BVE_2) as e:
            return self.exception_wrapper(e)

    def response_middleware(self, response):
        return response

    def response_wrapper(self, response):
        return HttpResponse(json.dumps({'status': 'ok', 'data': response}), content_type="application/json")

    def exception_wrapper(self, exception):
        response = {
            'status': exception.get_alias(),
            'message': exception.get_description(),
            'data': {}
        }
        if exception.get_field_problems():
            response['field_problems'] = exception.get_field_problems()
        http_code = exception.get_http_code()
        return HttpResponse(json.dumps(response), content_type="application/json", status=http_code)

    def get(self, *args, **kwargs):
        raise ServerError(ServerError.NOT_IMPLEMENTED)

    def post(self, *args, **kwargs):
        raise ServerError(ServerError.NOT_IMPLEMENTED)

    def put(self, *args, **kwargs):
        raise ServerError(ServerError.NOT_IMPLEMENTED)

    def patch(self, *args, **kwargs):
        raise ServerError(ServerError.NOT_IMPLEMENTED)

    def delete(self, *args, **kwargs):
        raise ServerError(ServerError.NOT_IMPLEMENTED)

    def head(self, *args, **kwargs):
        raise ServerError(ServerError.NOT_IMPLEMENTED)

    def options(self, *args, **kwargs):
        raise ServerError(ServerError.NOT_IMPLEMENTED)

    def trace(self, *args, **kwargs):
        raise ServerError(ServerError.NOT_IMPLEMENTED)


class CreateView(CsrfExemptMixin, PermissionsMixin, FormMixin, SerializerMixin, BaseView):
    def post(self, request, *args, **kwargs):
        self.check_w_permission(self.request.user)
        form = self.get_form()
        if form.is_valid():
            instance = form.save()
        else:
            raise FormException(form)
        return instance


class DetailsView(CsrfExemptMixin, PermissionsMixin, ObjectMixin, FormMixin, SerializerMixin, BaseView):
    def get(self, request, *args, **kwargs):
        self.check_r_permission(self.request.user)
        return self.get_object()

    def post(self, request, *args, **kwargs):
        self.check_w_permission(self.request.user)
        form = self.get_form()
        if form.is_valid():
            instance = form.save()
        else:
            raise FormException(form)
        return instance


class DeleteView(CsrfExemptMixin, PermissionsMixin, ObjectMixin, BaseView):
    def post(self, request, *args, **kwargs):
        self.check_d_permission(self.request.user)
        instance = self.get_object()
        instance.delete()
        return {}


class ListView(CsrfExemptMixin, PermissionsMixin, ListMixin, SerializerMixin, BaseView):
    def get(self, request, *args, **kwargs):
        self.check_r_permission(self.request.user)
        paginator = self.get_paginator()
        if paginator:
            instances = paginator.page()
        else:
            instances = self.get_queryset()
        return instances




import re
from os import path
from setuptools import setup, find_packages

BASE_DIR = path.abspath(path.dirname(__file__))


def find_version():
    for line in open(path.join(BASE_DIR, 'django_serializer/__init__.py')):
        if line.startswith('__version__'):
            return re.match(r"""__version__\s*=\s*(['"])([^'"]+)\1""", line).group(2)


setup(
    name='django-serializer',
    version=find_version(),
    description='Library for creating simple django api',
    long_description='Library for creating simple django api',

    author='Alexander Opryshko',
    author_email='alexopryshko@yandex.ru',
    url='https://github.com/alexopryshko/django-serializer',

    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='django-serializer setuptools development',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=[
        'Django>=1.10',
        'psycopg2>=2.6.2'
    ]
)



from django.db import models
from django.core.exceptions import ValidationError
from django.db.models.base import ModelBase

from django_serializer.exceptions import MissEntityTypeException, DuplicateEntityTypeException


ENTITY_TYPE_LENGTH = 16
ENTITY_ID_LENGTH = 64 - ENTITY_TYPE_LENGTH
_ENTITY_TYPE_MAX_VALUE = int('1' * ENTITY_TYPE_LENGTH, 2)
_ENTITY_ID_MAX_VALUE = int('1' * ENTITY_ID_LENGTH, 2)
_ENTITY_TYPE_MASK = _ENTITY_TYPE_MAX_VALUE << ENTITY_ID_LENGTH
_ENTITY_ID_MASK = ~_ENTITY_TYPE_MASK


class EntityMixinMeta(ModelBase):
    entity_types = {}

    def __new__(mcs, name, bases, attrs):
        if 'EntityMixin' == name or 'ENTITY_TYPE' not in attrs:
            new = super(EntityMixinMeta, mcs).__new__(mcs, name, bases, attrs)
            return new

        entity_type = attrs['ENTITY_TYPE']
        if entity_type is None:
            raise MissEntityTypeException('You have missed entity_type in {}'.format(name))

        if entity_type in mcs.entity_types:
            raise DuplicateEntityTypeException(
                'Model({}) with the same entity type already exist'.format(mcs.entity_types[entity_type].__name__)
            )

        attrs['entity_types'] = mcs.entity_types

        new = super(EntityMixinMeta, mcs).__new__(mcs, name, bases, attrs)
        mcs.entity_types[entity_type] = new

        return new

    def check_entity_type(cls, entity_type):
        return entity_type in cls.entity_types


class EntityMixin(metaclass=EntityMixinMeta):
    ENTITY_TYPE = None

    def get_entity_id(self):
        entity_type = int(self.ENTITY_TYPE)
        entity_id = int(self.pk)

        object_id = entity_type << ENTITY_ID_LENGTH
        object_id |= entity_id
        return object_id

    def get_entity_by_id(self, entity_id):
        if not entity_id:
            return
        entity_type, entity_id = split_object_id(entity_id)
        return self.entity_types[entity_type].objects.get(pk=entity_id)


class EntityField(models.BigIntegerField):
    @staticmethod
    def validate_object_id(value):
        if not check_object_id(value):
            raise ValidationError('%(value)s is not valid object id', params={'value': value})

    def __init__(self, *args, **kwargs):
        self.available_entities = kwargs.pop('available_entities', [])
        kwargs.update({
            'db_index': True,
            'validators': [self.validate_object_id]
        })
        super().__init__(*args, **kwargs)


def split_object_id(object_id):
    object_id = int(object_id)
    entity_type = (object_id & _ENTITY_TYPE_MASK) >> ENTITY_ID_LENGTH
    entity_id = object_id & _ENTITY_ID_MASK
    return entity_type, entity_id


def check_object_id(object_id, entity_type=None):
    try:
        object_id = int(object_id)
    except ValueError as e:
        return False

    object_entity_type, entity_id = split_object_id(object_id)

    if not EntityMixin.check_entity_type(object_entity_type):
        return False

    if entity_type and object_entity_type != entity_type:
        return False

    return True
